@@ -21,6 +21,8 @@ use glutin::dpi::{LogicalSize, PhysicalPosition, PhysicalSize};
 use glutin::event::{
     ElementState as GlutinElementState,
     Event as GlutinEvent,
+    MouseScrollDelta as GlutinMouseScrollDelta,
+    TouchPhase,
     VirtualKeyCode as GlutinVirtualKeyCode,
     WindowEvent as GlutinWindowEvent
 };
@@ -41,6 +43,7 @@ use crate::window::{
     EventLoopSendError,
     ModifiersState,
     MouseButton,
+    MouseScrollDistance,
     UserEventSender,
     VirtualKeyCode,
     WindowCreationError,
@@ -454,6 +457,31 @@ impl<UserEventType: 'static> WindowGlutin<UserEventType>
                     }
                 },
 
+                GlutinWindowEvent::MouseWheel {
+                    delta,
+                    phase: TouchPhase::Moved,
+                    ..
+                } => {
+                    let distance = match delta {
+                        GlutinMouseScrollDelta::LineDelta(x, y) => {
+                            MouseScrollDistance::Lines {
+                                x: x as f64,
+                                y: y as f64,
+                                z: 0.0
+                            }
+                        }
+                        GlutinMouseScrollDelta::PixelDelta(pos) => {
+                            MouseScrollDistance::Pixels {
+                                x: pos.x,
+                                y: pos.y,
+                                z: 0.0
+                            }
+                        }
+                    };
+
+                    handler.on_mouse_wheel_scroll(helper, distance);
+                }
+
                 GlutinWindowEvent::KeyboardInput { input, .. } => {
                     let virtual_key_code =
                         input.virtual_keycode.map(VirtualKeyCode::from);
