@@ -17,11 +17,12 @@
 use std::borrow::Borrow;
 use std::cell::{Cell, RefCell};
 use std::convert::TryInto;
-use std::ops::{Deref, DerefMut};
+use std::ops::{Deref, DerefMut, Mul};
 use std::rc::Rc;
 
 use wasm_bindgen::closure::Closure;
-use web_sys::{KeyboardEvent, MouseEvent};
+use wasm_bindgen::JsCast;
+use web_sys::{KeyboardEvent, MouseEvent, WheelEvent};
 
 use crate::dimen::Vector2;
 use crate::error::{BacktraceError, ErrorMessage};
@@ -33,6 +34,7 @@ use crate::window::{
     KeyScancode,
     ModifiersState,
     MouseButton,
+    MouseScrollDistance,
     UserEventSender,
     VirtualKeyCode,
     WindowFullscreenMode,
@@ -664,6 +666,8 @@ impl WebCanvasImpl
         let initial_size_scaled = canvas.html_element().element().dimensions();
         let initial_dpr = window.device_pixel_ratio();
 
+        let current_dpr = Rc::new(Cell::new(initial_dpr));
+
         let initial_size_unscaled =
             (initial_size_scaled * initial_dpr).round().into_u32();
 
@@ -778,6 +782,7 @@ impl WebCanvasImpl
             let helper = helper.clone();
             let window_inner = window.clone();
             let canvas = canvas.clone();
+            let current_dpr = current_dpr.clone();
 
             event_listeners_to_clean_up.push(
                 window
@@ -787,6 +792,8 @@ impl WebCanvasImpl
                         let size_scaled = canvas.html_element().element().dimensions();
                         let dpr = window_inner.device_pixel_ratio();
 
+                        Cell::replace(Rc::borrow(&current_dpr), dpr);
+
                         let size_unscaled = (size_scaled * dpr).round().into_u32();
 
                         canvas.set_buffer_dimensions(&size_unscaled);
@@ -849,16 +856,22 @@ impl WebCanvasImpl
         {
             let handler = handler.clone();
             let helper = helper.clone();
+            let current_dpr = current_dpr.clone();
 
             event_listeners_to_clean_up.push(
                 canvas_event_target.register_event_listener_mouse(
                     "mousemove",
                     move |event| {
+                        let current_dpr = Cell::get(Rc::borrow(&current_dpr)) as f32;
+
                         let position = if is_pointer_locked.get() {
                             Vector2::new(event.movement_x(), event.movement_y())
                                 .into_f32()
+                                .mul(current_dpr)
                         } else {
-                            Vector2::new(event.offset_x(), event.offset_y()).into_f32()
+                            Vector2::new(event.offset_x(), event.offset_y())
+                                .into_f32()
+                                .mul(current_dpr)
                         };
 
                         RefCell::borrow_mut(Rc::borrow(&handler)).on_mouse_move(
@@ -918,6 +931,50 @@ impl WebCanvasImpl
             );
         }
 
+        {
+            let handler = handler.clone();
+            let helper = helper.clone();
+
+            event_listeners_to_clean_up.push(
+                canvas_event_target.register_event_listener_mouse(
+                    "wheel",
+                    move |event| {
+                        let event: WheelEvent = event.dyn_into().unwrap();
+
+                        let delta = match event.delta_mode() {
+                            0x00 => MouseScrollDistance::Pixels {
+                                x: event.delta_x(),
+                                y: -event.delta_y(),
+                                z: event.delta_z()
+                            },
+
+                            0x01 => MouseScrollDistance::Lines {
+                                x: event.delta_x(),
+                                y: -event.delta_y(),
+                                z: event.delta_z()
+                            },
+
+                            0x02 => MouseScrollDistance::Pages {
+                                x: event.delta_x(),
+                                y: -event.delta_y(),
+                                z: event.delta_z()
+                            },
+
+                            mode => {
+                                log::error!("Mouse wheel: Unknown delta mode {}", mode);
+                                return;
+                            }
+                        };
+
+                        handler.borrow_mut().on_mouse_wheel_scroll(
+                            helper.borrow_mut().deref_mut(),
+                            delta
+                        );
+                    }
+                )?
+            );
+        }
+
         let modifier_state = Rc::new(RefCell::new(ModifiersState::default()));
 
         {
@@ -989,6 +1046,8 @@ impl WebCanvasImpl
                     let new_dpr = window.device_pixel_ratio();
                     log::info!("DPI changed to {}", new_dpr);
 
+                    Cell::replace(Rc::borrow(&current_dpr), new_dpr);
+
                     handler.borrow_mut().on_scale_factor_changed(
                         RefCell::borrow_mut(Rc::borrow(&helper)).deref_mut(),
                         new_dpr
